/******************************************************************************
 Example to update the flash configuration record 
*****************************************************************************/
#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <netinterface.h>
#include <system.h>
#include <string.h>   // needed for memcpy()
#include <bsp.h>


#define FLAG_USE_WD (1) // Bit0 = enable/disable watchdog in Mod5282

extern "C" { 
   void UserMain(void * pd);
}

const char * AppName="UpdateConfig";


/* The gConfigRecord in \nburn\include\system.h is copied below, per tools revisoin 
   1.96 RC1. The gConfigRecord is defined as a global for the application, so the
   strucute is commented out here and used for reference only.

   typedef struct
   {
         unsigned long recordsize;           //   0 The stored size of the struct 
         unsigned long ip_Addr;              //   4 The device IP Address 
         unsigned long ip_Mask;              //   8 The IP Address Mask 
         unsigned long ip_GateWay;           //  12 The address of the P gateway 
         unsigned long ip_TftpServer;        //  16 The address of the TFTP server to load data from for debugging 
         unsigned long baud_rate;            //  20 The initial system Baud rate 
         unsigned char wait_seconds;         //  24 The number of seconds to wait before booting 
         unsigned char bBoot_To_Application; //  25 True if we boot to the application, not the monitor 
         unsigned char bException_Action;    //  26 What should we do when we have an exception? 
         unsigned char m_FileName[80];       //  27 The file name of the TFTP file to load 
         unsigned char mac_address[6];       // 107 The Ethernet MAC address 
         unsigned char ser_boot;             // 113 Boot serial port: 0, 1 or 2
         unsigned long ip_DNS_server;        // 114 DNS server IP address
         unsigned char core_mac_address[6];  // 118 The Base unit MAC address
         unsigned char typeof_if;            // 124 Network interface type
         unsigned char direct_Tx;            // 125
         unsigned long m_ExtraData[4];       // 126 Platform specific data
         unsigned char m_bUnused[3];         // 142 Reserved data space
         unsigned char m_q_boot;             // 145 True to boot without debug messages
         unsigned short checksum;            // 146 A Checksum for this structure 
   }__attribute__(( packed) ) ConfigRecord;

*/


/* The structure used by the Mod5282 monitor to access the system configuration space
   in flash is as follows:
*/
typedef struct
{                                          // Starting offest
      unsigned long recordsize;            //   0  The stored size of the struct
      unsigned long ip_Addr;               //   4  The device IP Address 
      unsigned long ip_Mask;               //   8  The IP Address Mask 
      unsigned long ip_GateWay;            //  12  The address of the P gateway
      unsigned long ip_TftpServer;         //  16  TFTP server IP address
      unsigned long baud_rate;             //  20  The initial system Baud rate 
      unsigned char wait_seconds;          //  24  The number of seconds to wait before booting 
      unsigned char bBoot_To_Application;  //  25  True if we boot to the application, not the monitor 
      unsigned char bException_Action;     //  26  What should we do when we have an exception? 
      unsigned char m_FileName[80];        // 106  The file name of the TFTP file to load 
      unsigned char mac_address[6];        // 112  The Ethernet MAC address 
      unsigned char ser_port;              // 113  Boot port selection
      unsigned long ip_DNS_Server;         // 114  The System DNS server address 
      unsigned long m_Unused[6];           // 118  Reserved space
      unsigned char m_bUnused[2];          // 142  Reserved space
	  unsigned char m_Flags;               // 144  Hardware specific function flags
      unsigned char q_boot;                // 145  Quite boot mode
      unsigned short checksum;             // 146  Checksum for this structure
}__attribute__(( packed)) MonitorConfigRecord; 


MonitorConfigRecord BootConfigRec;



/******************************************************************************
 UserMain
*****************************************************************************/
void UserMain(void * pd)
{
   InitializeStack();
   int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf( "\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
      iprintf("Static IP Address: ");
   ShowIP(ib->netIP); iprintf("\r\n");

   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();

   iprintf("sizeof gConfigRec: %d\r\n", sizeof(gConfigRec) );
   iprintf("gConfigRec.recordsize: %d\r\n", gConfigRec.recordsize );
   iprintf("sizeof BootConfigRec: %d\r\n", sizeof(BootConfigRec) );

   // From the MonitorconfigRecord we can see that the m_Flags bit mask is
   // at an offset of 144 bytes. The watchdog enable/disable is controlled
   // by bit0 of m_Flags. The corresponding offset of 144 bytes in the
   // gConfigRec is m_bUnused[2]. At this point there are 2 choices: 
   // map the flash space to the MonitorConfigRecord structure and 
   // reference m_Flags, or use the gConfigRec and reference m_bUnused[2].
   // For the purposes of this example, we will use the later method.

   iprintf("m_bUnused[2] = 0x%02X", gConfigRec.m_bUnused[2]);
   if ( gConfigRec.m_bUnused[2] & FLAG_USE_WD )
      iprintf(" Watchdog Enabled (and will time out)\r\n");
   else
      iprintf(" Watchdog Disabled\r\n");
   
   iprintf("Hit any key to enable the watchdog\r\n");
   ConfigRecord cr;
   memcpy((PBYTE) &cr, (PBYTE) &gConfigRec, sizeof(ConfigRecord));
   
   while (1)
   {
      char c = getchar();
      if ( !(gConfigRec.m_bUnused[2] & FLAG_USE_WD) )  
         cr.m_bUnused[2] |= FLAG_USE_WD;    // enable watchdog

      // Note: you cannot try to disable the watchdog using this
      // method, since the watchdog reset could occur during the
      // flash update process and corrupt the configuration record.
      // To disable the watchdog you must use the serial port
      // interface, enter the setup menu, and disable the watchdog
      // from there. Whenever you enter the monitor program, the
      // watchdog is disabled, so the config record can be
      // written without error.

      UpdateConfigRecord(&cr);
      ForceReboot();
      
      OSTimeDly(20);
   }
}


